/**
 * @author 龙喜<xiaolong.lxl@alibaba-inc.com>
 * @description 构建配置加载器
 */

'use strict';

const fs = require('fs');
const path = require('path');
const config = require('../config');
const Debug = require('debug');
const debug = Debug('builder:rendererLoader');

module.exports = {
  /**
   * 获取构建配置
   * @returns {*}
   */
  get renderers() {
    if (!this.availableRenderers) {
      // 用户开启的 renderers
      let userActiveRenderers = config.renderers;
      // 最终可用 renderers
      this.availableRenderers = fs.readdirSync(path.join(__dirname, '../renderers'))
        .filter(function(fileName) {
          return userActiveRenderers.indexOf(fileName) !== -1;
        })
        .map(function(fileName) {
          return path.join(__dirname, '../renderers', fileName);
        })
        .filter(function(filePath) {
          return fs.statSync(filePath).isDirectory();
        }).map(function(filePath) {
          return require(filePath);
        });
    }

    return this.availableRenderers;
  }
};
